﻿using System;
using System.Net;
using System.Net.Sockets;
using AMP;

namespace Simple_Server
{
    class Server
    {
        static void Main(string[] args)
        {
            int port = 1234;
            var socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            socket.Bind(new IPEndPoint(IPAddress.Parse("127.0.0.1"), port));
            socket.Listen(5);
            Console.WriteLine("Listening on port {0}", port);

            AMPSocketTransport transport;
            AMP_Protocol protocol;
            Socket clientSocket;
            while (true)
            {
                clientSocket = socket.Accept();
                transport = new AMPSocketTransport(clientSocket);
                protocol = new AMP_Protocol(transport);

                // we could also pass a 'state' argument to RegisterResponder
                // which becomes the 'state' argument in handleSum (below).
                // Since we aren't passing anything, then state will always be null
                protocol.RegisterResponder(new Sum(), handleSum);
                transport.Start(); // start the data-reading loop
            }

        }

        static Object handleSum(AMP_Msg reqMsg, Object state)
        {
            int a = (int)reqMsg["a"];
            int b = (int)reqMsg["b"];
            int total = a + b; 
            Console.WriteLine("Did a sum: {0} + {1} = {2}", a, b, total);
            return new AMP_Msg {{"total", total}};
        }
    }

    public class Sum: AMP_Command
    {
        public Sum()
            : base("Sum")
        {
            arguments.Add(new AMP_Parameter("a", new AMP_Integer()));
            arguments.Add(new AMP_Parameter("b", new AMP_Integer()));

            response.Add(new AMP_Parameter("total", new AMP_Integer()));
        }
    }
}
